#include "menu.h"
#include "config.h"
#include "gfxinterface.h"
#include "input.h"
#include "time_util.h"


const char * menu_lines[] = {
	"XGalaga++",
	"v0.3 (2003-10-10)",
	"",
	"2003 Marc Mongenet, pictures from XGalaga 2.0.34",
	"This game is free software, covered by the GPL.",
	"",
	"Key             Action",
	"             ",
	"s               start",
	"q               quit",
	"p               pause",
	"1,2,3,4         standard window size",
	"+               increase details (slower)",
	"-               decrease details (faster)",
	"<space bar>     fire",
	"<left arrow>    move left",
	"<right arrow>   move right",
};


StartMenu * StartMenu::singleton_ (0);


StartMenu::StartMenu()
: font_(X11::Inst().LoadQueryFont("*-fixed-bold-r-*"))
, width_(0)
{
	if (!font_) throw std::runtime_error ("X font error");

	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		width_ = std::max(width_, XTextWidth(font_, menu_lines[i],
		                                     std::strlen(menu_lines[i])));
	}
}


StartMenu::~StartMenu()
{
	X11::Inst().FreeFont(font_);
}


void SetWindowSize(unsigned win_size)
{
	static const unsigned widths[]  = { 640, 800, 1024, 1280 };
	static const unsigned heights[] = { 480, 600, 768, 1024 };
	if (win_size > 0 && win_size <= sizeof widths / sizeof widths[0]) {
		--win_size;
		X11::Inst().ResizeWindow(widths[win_size], heights[win_size]);
		X11::Inst().ClearWindow();
	}
}


bool StartMenu::Display(std::vector<StarsField> * stars_fields, Score * score) const
{
	Input input;
        for (;;) {
		const double start_frame_time (CurrentMicroSecond());
		input.Update();
		if (input.Quit()) return false;
		if (input.Start()) return true;
		SetWindowSize(input.WindowSize());
		Details::Instance().Change(input.Details());
		X11::Inst().GetWindowAttributes();
		ScrollStarFields(stars_fields);
		score->Refresh();
		Print();
		X11::Inst().Sync(False);
		WaitNextFrame(start_frame_time);
	}
}


void StartMenu::Print() const
{
	const int left (std::max(0, (X11::Inst().WindowWidth() - width_) / 2));
	Coord pos (left ,20);
	X11::Inst().SetForeground(X11::Inst().GetWhite());
	X11::Inst().SetFont(font_->fid);
	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		X11::Inst().DrawString(pos, menu_lines[i]);
		pos.y += font_->ascent + font_->descent + 1;
	}
}
